# NOTE: This file is auto-generated by "inv sync_meta"

name = "gutt"
version = "0.6.0"
description = "Auto generate unit test templates"
authors = ["Ryan <ryanchao2012@gmail.com>"]
license = "MIT"
readme = "README.md"
repository = "https://github.com/ryanchao2012/gutt"
